@@ -25,6 +25,7 @@ import sys
 import re
 import unicodedata
 import traceback
+from collections import defaultdict
 from PyQt4 import QtCore
 from picard.track import Track
 from picard.mbxml import artist_credit_from_node
@@ -138,7 +139,7 @@ class File(LockableObject, Item):
     def copy_metadata(self, metadata):
         exceptions = ['musicip_puid', 'acoustid_id']
         if self.config.setting['preserved_tags']:
-            exceptions.extend(re.split(r'\s+', self.config.setting['preserved_tags'].lower().strip()))
+            exceptions.extend(re.split(r'\s+', self.config.setting['preserved_tags'].strip()))
         for tag in exceptions:
             self.saved_metadata[tag] = self.metadata[tag]
         self.metadata.copy(metadata)
@@ -197,7 +198,7 @@ class File(LockableObject, Item):
                 pass
         # Save cover art images
         if settings["save_images_to_files"]:
-            self._save_images(new_filename, metadata, settings)
+            self._save_images(os.path.dirname(new_filename), metadata, settings)
         return new_filename
 
     @staticmethod
@@ -312,37 +313,47 @@ class File(LockableObject, Item):
         else:
             return old_filename
 
-    def _save_images(self, filename, metadata, settings):
-        """Save the cover images to disk."""
-        if not metadata.images:
-            return
-        overwrite = settings["save_images_overwrite"]
-        image_filename = self._script_to_filename(
-            settings["cover_image_filename"], metadata, settings)
+    def _make_image_filename(self, image_filename, dirname, metadata, settings):
+        image_filename = self._script_to_filename(image_filename, metadata, settings)
         if not image_filename:
             image_filename = "cover"
         if os.path.isabs(image_filename):
             filename = image_filename
         else:
-            filename = os.path.join(os.path.dirname(filename), image_filename)
+            filename = os.path.join(dirname, image_filename)
         if settings['windows_compatible_filenames'] or sys.platform == 'win32':
             filename = filename.replace('./', '_/').replace('.\\', '_\\')
-        filename = encode_filename(filename)
-        i = 0
-        for mime, data in metadata.images:
+        return encode_filename(filename)
+
+    def _save_images(self, dirname, metadata, settings):
+        """Save the cover images to disk."""
+        if not metadata.images:
+            return
+        default_filename = self._make_image_filename(
+            settings["cover_image_filename"], dirname, metadata, settings)
+        overwrite = settings["save_images_overwrite"]
+        counters = defaultdict(lambda: 0)
+        for mime, data, filename in metadata.images:
+            if filename is None:
+                filename = default_filename
+            else:
+                filename = self._make_image_filename(filename, dirname, metadata, settings)
             image_filename = filename
             ext = mimetype.get_extension(mime, ".jpg")
-            if i > 0:
-                image_filename = "%s (%d)" % (filename, i)
-            i += 1
+            if counters[filename] > 0:
+                image_filename = "%s (%d)" % (filename, counters[filename])
+            counters[filename] = counters[filename] + 1
             while os.path.exists(image_filename + ext) and not overwrite:
                 if os.path.getsize(image_filename + ext) == len(data):
                     self.log.debug("Identical file size, not saving %r", image_filename)
                     break
-                image_filename = "%s (%d)" % (filename, i)
-                i += 1
+                image_filename = "%s (%d)" % (filename, counters[filename])
+                counters[filename] = counters[filename] + 1
             else:
                 self.log.debug("Saving cover images to %r", image_filename)
+                new_dirname = os.path.dirname(image_filename)
+                if not os.path.isdir(new_dirname):
+                    os.makedirs(new_dirname)
                 f = open(image_filename + ext, "wb")
                 f.write(data)
                 f.close()
@@ -466,6 +477,13 @@ class File(LockableObject, Item):
         if hasattr(file.info, 'bits_per_sample') and file.info.bits_per_sample:
             metadata['~#bits_per_sample'] = file.info.bits_per_sample
         metadata['~format'] = self.__class__.__name__.replace('File', '')
+        self._add_path_to_metadata(metadata)
+
+    def _add_path_to_metadata(self, metadata):
+        metadata['~dirname'] = os.path.dirname(self.filename)
+        filename = os.path.basename(self.filename)
+        metadata['~filename'], metadata['~extension'] = os.path.splitext(
+                os.path.basename(self.filename))
 
     def get_state(self):
         return self._state
