@@ -247,7 +247,7 @@ class ID3File(File):
             tags.add(id3.TPOS(encoding=0, text=text))
 
         if settings['save_images_to_tags']:
-            for mime, data in metadata.images:
+            for mime, data, _fname in metadata.images:
                 tags.add(id3.APIC(encoding=0, mime=mime, type=3, desc='', data=data))
 
         tmcl = mutagen.id3.TMCL(encoding=encoding, people=[])
