@@ -42,8 +42,8 @@ class Metadata(object):
         self.images = []
         self.length = 0
 
-    def add_image(self, mime, data):
-        self.images.append((mime, data))
+    def add_image(self, mime, data, filename=None):
+        self.images.append((mime, data, filename))
 
     def __repr__(self):
         return repr(self._items)
@@ -199,6 +199,10 @@ class Metadata(object):
         if value or value == 0:
             self._items.setdefault(name, []).append(value)
 
+    def add_unique(self, name, value):
+        if value not in self.getall(name):
+            self.add(name, value)
+
     def keys(self):
         return self._items.keys()
 
