@@ -68,8 +68,6 @@ from picard.acoustidmanager import AcoustIDManager
 from picard.util import (
     decode_filename,
     encode_filename,
-    sanitize_filename,
-    icontheme,
     partial,
     queue,
     thread,
@@ -326,9 +324,9 @@ class Tagger(QtGui.QApplication):
                         self.move_file_to_album(file, albumid)
                 elif mbid_validate(trackid):
                     self.move_file_to_nat(file, trackid)
-                elif self.config.setting['analyze_new_files']:
+                elif self.config.setting['analyze_new_files'] and file.can_analyze():
                     self.analyze([file])
-            elif self.config.setting['analyze_new_files']:
+            elif self.config.setting['analyze_new_files'] and file.can_analyze():
                 self.analyze([file])
 
     def add_files(self, filenames):
@@ -458,12 +456,6 @@ class Tagger(QtGui.QApplication):
         album.load()
         return album
 
-    def reload_album(self, album):
-        if album == self.nats:
-            album.update()
-        else:
-            album.load()
-
     def load_nat(self, id, node=None):
         self.create_nats()
         nat = self.get_nat_by_id(id)
@@ -632,10 +624,7 @@ class Tagger(QtGui.QApplication):
 
     def refresh(self, objs):
         for obj in objs:
-            if isinstance(obj, Album):
-                self.reload_album(obj)
-            elif isinstance(obj, NonAlbumTrack):
-                obj.load()
+            obj.load()
 
     @classmethod
     def instance(cls):
