@@ -122,7 +122,7 @@ class MainWindow(QtGui.QMainWindow):
     def keyPressEvent(self, event):
         if event.matches(QtGui.QKeySequence.Delete):
             if self.metadata_box.hasFocus():
-                self.metadata_box.remove_selected_tag()
+                self.metadata_box.remove_selected_tags()
             else:
                 self.remove()
         else:
@@ -207,7 +207,7 @@ class MainWindow(QtGui.QMainWindow):
 
     def create_statusbar(self):
         """Creates a new status bar."""
-        self.statusBar().showMessage("Ready")
+        self.statusBar().showMessage(_("Ready"))
         self.file_counts_label = QtGui.QLabel()
         self.listening_label = QtGui.QLabel()
         self.listening_label.setVisible(False)
@@ -644,7 +644,7 @@ been merged with that of single artist albums."""),
         webbrowser2.open("http://forums.musicbrainz.org/viewforum.php?id=2")
 
     def open_donation_page(self):
-        webbrowser2.open('http://metabrainz.org/donate/index.html')
+        webbrowser2.open('http://metabrainz.org/donate')
 
     def save(self):
         """Tell the tagger to save the selected objects."""
